local addon = ...

-------------
-- Main Interface Window Builder
-------------
local AnnounceAdd = {};
AnnounceAdd.panel = CreateFrame( "Frame", "AnnounceAdd", UIParent );	-- Create the Panel
AnnounceAdd.panel.name = "Paragon Announcer";		-- Name the Panel
InterfaceOptions_AddCategory(AnnounceAdd.panel);	-- Add to the Interface Options
AnnounceAdd.panel:RegisterEvent("ADDON_LOADED")		-- Trigger an event when this addon loads.
AnnounceAdd.panel:SetScript("OnEvent", function(self, event, ...)
	if (event == "ADDON_LOADED") then
		-- Register Generic Addon Settings
		AnnounceAdd:RegisterDefaultGenericSetting("ShowAlert", true) -- Should we show an alert window?
		AnnounceAdd:RegisterDefaultGenericSetting("ShowChat", true) -- Should we show a chat message?
		AnnounceAdd:RegisterDefaultGenericSetting("NullCheckBox", false) -- This check box does nothing. We should save it anyway.
	
		-- Register Legion Paragon Settings
		AnnounceAdd:RegisterDefaultLegionSetting("46743", true, "Highmountain") -- Supplies From Highmountain
		AnnounceAdd:RegisterDefaultLegionSetting("46745", true, "Court of Farondis") -- Supplies From the Court
		AnnounceAdd:RegisterDefaultLegionSetting("46746", true, "Valajar") -- Supplies From the Valarjar
		AnnounceAdd:RegisterDefaultLegionSetting("46747", true, "Dreamweavers") -- Supplies From the Dreamweavers
		AnnounceAdd:RegisterDefaultLegionSetting("46748", true, "Nightfallen")	-- Supplies From the Nightfallen
		AnnounceAdd:RegisterDefaultLegionSetting("46749", true, "Wardens Supplies")	-- Supplies From the Wardens
		AnnounceAdd:RegisterDefaultLegionSetting("46777", true, "Legionfall") -- The Bounties of Legionfall
		AnnounceAdd:RegisterDefaultLegionSetting("46800", true, "Wardens (old)") -- Paragon of the Wardens
		AnnounceAdd:RegisterDefaultLegionSetting("48976", true, "Argussian Reach") -- Supplies From the Argussian Reach
		AnnounceAdd:RegisterDefaultLegionSetting("48977", true, "Army of the Light") -- Supplies From the Army of the Light
		
		-- Register BfA Paragon Settings
		AnnounceAdd:RegisterDefaultBfASetting("54453", true, "Champions of Azeroth") -- Supplies from Magni
		AnnounceAdd:RegisterDefaultBfASetting("54451", true, "Tortollan Seekers") -- Baubles from the Seekers
		AnnounceAdd:RegisterDefaultBfASetting("55348", true, "Rustbolt Resistance") -- Supplies from the Rustbolt Resistance
		AnnounceAdd:RegisterDefaultBfASetting("58096", true, "Rajani") -- Supplies from the Rajani
		AnnounceAdd:RegisterDefaultBfASetting("58097", true, "Uldum Accord") -- Supplies from the Uldum Accord
		AnnounceAdd:RegisterDefaultBfASetting("54454", true, "7th Legion") -- Supplies from the 7th Legion
		AnnounceAdd:RegisterDefaultBfASetting("54456", true, "Order of Embers") -- Supplies from the Order of Embers
		AnnounceAdd:RegisterDefaultBfASetting("54457", true, "Storm's Wake") -- Supplies from Storm's Wake
		AnnounceAdd:RegisterDefaultBfASetting("54458", true, "Proudmoore Admiralty") -- Supplies from the Proudmoore Admiralty
		AnnounceAdd:RegisterDefaultBfASetting("55976", true, "Waveblade Ankoan") -- Supplies From the Waveblade Ankoan
		AnnounceAdd:RegisterDefaultBfASetting("54455", true, "The Honorbound") -- Supplies from the Honorbound
		AnnounceAdd:RegisterDefaultBfASetting("54461", true, "Voldunai") -- Supplies from the Voldunai
		AnnounceAdd:RegisterDefaultBfASetting("54462", true, "Zandalari Empire") -- Supplies from the Zandalari Empire
		AnnounceAdd:RegisterDefaultBfASetting("54460", true, "Talanji's Expedition") -- Supplies from Talanji's Expedition
		AnnounceAdd:RegisterDefaultBfASetting("53982", true, "The Unshackled") -- Supplies From The Unshackled
		
		-- Register Shadowlands Paragon Settings
		--AnnounceAdd:RegisterDefaultShadowlandsSetting("46293", true, "Test Function") -- Test function.
		AnnounceAdd:RegisterDefaultShadowlandsSetting("61097", true, "The Ascended") -- Supplies from the Ascended
		AnnounceAdd:RegisterDefaultShadowlandsSetting("61100", true, "Court of Harvesters") -- Baubles from the Court of Harvesters
		AnnounceAdd:RegisterDefaultShadowlandsSetting("61095", true, "Undying Army") -- Supplies from the Undying Army
		AnnounceAdd:RegisterDefaultShadowlandsSetting("61098", true, "The Wild Hunt") -- Supplies from the Wild Hunt
		AnnounceAdd:RegisterDefaultShadowlandsSetting("64267", true, "Ve'nari") -- Mysterious Gifts from Venari
		AnnounceAdd:RegisterDefaultShadowlandsSetting("64012", true, "Death's Advance") -- Supplies from Death's Advance
		AnnounceAdd:RegisterDefaultShadowlandsSetting("64266", true, "Archivist's Codex") -- Supplies from Archivist's Codex

		AnnounceAdd.panel:UnregisterEvent("ADDON_LOADED")
    end
end)

-------------
-- Checkbox Creation function used throughout the AddOn.
-------------
function AnnounceAdd:CreateCheckBox(name, parent, label, tooltip, relativeTo, x, y, disableInCombat)
    local checkBox = CreateFrame("CheckButton", name, parent, "InterfaceOptionsCheckButtonTemplate")
    checkBox:SetPoint("TOPLEFT", relativeTo, "BOTTOMLEFT", x, y)
    checkBox.Text:SetText(label)

    if (tooltip) then
        checkBox.tooltipText = tooltip
    end

    return checkBox
end

-------------
-- Generic Registration Panel
-------------
function AnnounceAdd:RegisterDefaultGenericSetting(key, value)
    if (ParagonSettings == nil) then
        ParagonSettings = {}
    end
   if (ParagonSettings[key] == nil) then
        ParagonSettings[key] = value
   end
end

-------------
-- Main Interface Window Checkboxes
-------------
AnnounceAdd.panel:Hide()
AnnounceAdd.panel:SetScript("OnShow", function()
    local Title = AnnounceAdd.panel:CreateFontString("Title", "ARTWORK", "GameFontNormalLarge")
    Title:SetPoint("TOPLEFT", AnnounceAdd.panel, 16, -16)
	Title:SetText("Paragon Announcer")

	local ShowAlert = AnnounceAdd:CreateCheckBox("ShowAlert", AnnounceAdd.panel, "Show Alerts", nil, Title, 0, -30, false)
    ShowAlert:SetScript("OnClick", function(self)
        local checked = not not self:GetChecked()
        PlaySound(SOUNDKIT.IG_MAINMENU_OPTION_CHECKBOX_ON)
		ParagonSettings.ShowAlert = checked
    end)

    local ShowChat = AnnounceAdd:CreateCheckBox("ShowChat", AnnounceAdd.panel, "Show Chat Messages", nil, ShowAlert, 0, -5, false)
    ShowChat:SetScript("OnClick", function(self)
        local checked = not not self:GetChecked()
        PlaySound(SOUNDKIT.IG_MAINMENU_OPTION_CHECKBOX_ON)
        ParagonSettings.ShowChat = checked
    end)

    local NullCheckBox = AnnounceAdd:CreateCheckBox("NullCheckBox", AnnounceAdd.panel, "This Checkbox Intentionally Left Blank", nil, ShowChat, 0, -5, false)
    NullCheckBox:SetScript("OnClick", function(self)
        local checked = not not self:GetChecked()
        PlaySound(SOUNDKIT.IG_MAINMENU_OPTION_CHECKBOX_ON)
        ParagonSettings.NullCheckBox = checked
    end)

    function AnnounceAdd:Refresh()
        ShowAlert:SetChecked(ParagonSettings.ShowAlert)
        ShowChat:SetChecked(ParagonSettings.ShowChat)
        NullCheckBox:SetChecked(ParagonSettings.NullCheckBox)
    end

    AnnounceAdd:Refresh()
    AnnounceAdd.panel:SetScript("OnShow", nil)
end)

-------------
-- Legion Child Panel
-------------
 AnnounceAdd.childpanelLegion = CreateFrame( "Frame", "MyAddonChild", AnnounceAdd.panel);	-- Create the panel
 AnnounceAdd.childpanelLegion.name = "Legion";	-- Name the Panel
 AnnounceAdd.childpanelLegion.parent = AnnounceAdd.panel.name; -- Shows that this is a child panel instead of top level
 InterfaceOptions_AddCategory(AnnounceAdd.childpanelLegion); -- Adds to the Interface Window
 
-------------
-- Legion Registration Panel
-------------
function AnnounceAdd:RegisterDefaultLegionSetting(key, value, qName)
	if (QuestNames == nil) then
		QuestNames = {}
	end
	if (QuestNames[key] == nil) then
        QuestNames[key] = qName
   end
    if (QuestTableLegion == nil) then
        QuestTableLegion = {}
    end
   if (QuestTableLegion[key] == nil) then
        QuestTableLegion[key] = value
   end
end

-------------
-- Legion Checkbox Functions
-------------
AnnounceAdd.childpanelLegion:Hide()
AnnounceAdd.childpanelLegion:SetScript("OnShow", function()
    local Title = AnnounceAdd.childpanelLegion:CreateFontString("Title", "ARTWORK", "GameFontNormalLarge")
    Title:SetPoint("TOPLEFT", AnnounceAdd.childpanelLegion, 16, -16)
	Title:SetText("Paragon Announcer - Legion")
	
	local lastPanel = Title
	local buttonStorage = {}
	for key,value in pairs(QuestTableLegion) do
		local title, level, _, _, _, _, _, questID = C_QuestLog.GetTitleForLogIndex(key)
		local questKey = AnnounceAdd:CreateCheckBox(key, AnnounceAdd.childpanelLegion, QuestNames[key], nil, lastPanel, 0, -10, false)
		questKey:SetScript("OnClick", function(self)
			local checked = not not self:GetChecked()
			PlaySound(SOUNDKIT.IG_MAINMENU_OPTION_CHECKBOX_ON)
			QuestTableLegion[key] = checked
		end)
		lastPanel = questKey
		buttonStorage[key] = questKey
	end
	
	function AnnounceAdd:Refresh()
		for key,value in pairs(buttonStorage) do
			value:SetChecked(QuestTableLegion[key])
		end
    end

    AnnounceAdd:Refresh()
    AnnounceAdd.childpanelLegion:SetScript("OnShow", nil)
end)

 
-------------
-- Battle Child Panel
-------------
 AnnounceAdd.childpanelBattle = CreateFrame( "Frame", "MyAddonChild", AnnounceAdd.panel);
 AnnounceAdd.childpanelBattle.name = "Battle for Azeroth";
 AnnounceAdd.childpanelBattle.parent = AnnounceAdd.panel.name; -- Shows that this is a child panel instead of top level
 InterfaceOptions_AddCategory(AnnounceAdd.childpanelBattle); -- Adds to the Interface Window

-------------
-- Battle Registration Panel
-------------
function AnnounceAdd:RegisterDefaultBfASetting(key, value, qName)
	if (QuestNames == nil) then
		QuestNames = {}
	end
	if (QuestNames[key] == nil) then
        QuestNames[key] = qName
   end
    if (QuestTableBFA == nil) then
        QuestTableBFA = {}
    end
   if (QuestTableBFA[key] == nil) then
        QuestTableBFA[key] = value
   end
end

function AnnounceAdd:CreateCheckBox(name, parent, label, tooltip, relativeTo, x, y, disableInCombat)
    local checkBox = CreateFrame("CheckButton", name, parent, "InterfaceOptionsCheckButtonTemplate")
    checkBox:SetPoint("TOPLEFT", relativeTo, "BOTTOMLEFT", x, y)
    checkBox.Text:SetText(label)

    if ( tooltip ) then
        checkBox.tooltipText = tooltip
    end

    return checkBox
end

-------------
-- Battle Checkbox Functions
-------------
AnnounceAdd.childpanelBattle:Hide()
AnnounceAdd.childpanelBattle:SetScript("OnShow", function()

    local Title = AnnounceAdd.childpanelBattle:CreateFontString("Title", "ARTWORK", "GameFontNormalLarge")
    Title:SetPoint("TOPLEFT", AnnounceAdd.childpanelBattle, 16, -16)
	Title:SetText("Paragon Announcer - BfA")
	
	local lastPanel = Title		-- Stores our last panel for alignment use.
	local buttonStorage = {}	-- Stores our checkboxes for updated calls.
	for key,value in pairs(QuestTableBFA) do
		local title, level, _, _, _, _, _, questID = C_QuestLog.GetTitleForLogIndex(key)
		local questKey = AnnounceAdd:CreateCheckBox(key, AnnounceAdd.childpanelBattle, QuestNames[key], nil, lastPanel, 0, -10, false)
		questKey:SetScript("OnClick", function(self)
			local checked = not not self:GetChecked()
			PlaySound(SOUNDKIT.IG_MAINMENU_OPTION_CHECKBOX_ON)
			QuestTableBFA[key] = checked
		end)
		lastPanel = questKey
	 	buttonStorage[key] = questKey
	end
	
	function AnnounceAdd:Refresh()
		for key,value in pairs(buttonStorage) do
			value:SetChecked(QuestTableBFA[key])
		end
	end

    AnnounceAdd:Refresh()
    AnnounceAdd.childpanelBattle:SetScript("OnShow", nil)
end)

-------------
-- Shadowlands Child Panel
-------------
 AnnounceAdd.childpanelShadowlands = CreateFrame( "Frame", "MyAddonChild", AnnounceAdd.panel);
 AnnounceAdd.childpanelShadowlands.name = "Shadowlands";
 AnnounceAdd.childpanelShadowlands.parent = AnnounceAdd.panel.name; -- Shows that this is a child panel instead of top level
 InterfaceOptions_AddCategory(AnnounceAdd.childpanelShadowlands); -- Adds to the Interface Window

-------------
-- Shadowlands Registration Panel
-------------
function AnnounceAdd:RegisterDefaultShadowlandsSetting(key, value, qName)
	if (QuestNames == nil) then
		QuestNames = {}
	end
	if (QuestNames[key] == nil) then
        QuestNames[key] = qName
   end
    if (QuestTableShadowlands == nil) then
        QuestTableShadowlands = {}
    end
   if (QuestTableShadowlands[key] == nil) then
        QuestTableShadowlands[key] = value
   end
end

-------------
-- Shadow Checkbox Functions
-------------
AnnounceAdd.childpanelShadowlands:Hide()
AnnounceAdd.childpanelShadowlands:SetScript("OnShow", function()

    local Title = AnnounceAdd.childpanelShadowlands:CreateFontString("Title", "ARTWORK", "GameFontNormalLarge")
    Title:SetPoint("TOPLEFT", AnnounceAdd.childpanelShadowlands, 16, -16)
	Title:SetText("Paragon Announcer - Shadowlands")
	
	local lastPanel = Title		-- Stores our last panel for alignment use.
	local buttonStorage = {}	-- Stores our checkboxes for updated calls.
	for key,value in pairs(QuestTableShadowlands) do
		local title, level, _, _, _, _, _, questID = C_QuestLog.GetTitleForLogIndex(key)
		local questKey = AnnounceAdd:CreateCheckBox(key, AnnounceAdd.childpanelShadowlands, QuestNames[key], nil, lastPanel, 0, -10, false)
		questKey:SetScript("OnClick", function(self)
			local checked = not not self:GetChecked()
			PlaySound(SOUNDKIT.IG_MAINMENU_OPTION_CHECKBOX_ON)
			QuestTableShadowlands[key] = checked
		end)
		lastPanel = questKey
	 	buttonStorage[key] = questKey
	end
	
	function AnnounceAdd:Refresh()
		for key,value in pairs(buttonStorage) do
			value:SetChecked(QuestTableShadowlands[key])
		end
	end

    AnnounceAdd:Refresh()
    AnnounceAdd.childpanelShadowlands:SetScript("OnShow", nil)
end)

-------------
-- Quest Cache Function.  Use this to let the player know that they have a new rep cache waiting for them.  
-------------
local QuestFrame = CreateFrame("Frame")
QuestFrame:RegisterEvent("QUEST_ACCEPTED")
QuestFrame:SetScript("OnEvent", function(self, event, questIndex)
    local title = C_QuestLog.GetTitleForQuestID(questIndex)
    if QuestTableLegion[tostring(questIndex)] or QuestTableBFA[tostring(questIndex)] or QuestTableShadowlands[tostring(questIndex)] then
		local text = format("New Cache Quest added to your log:\n" ..title.. "\n\nID #:" ..questIndex)
        if ParagonSettings.ShowAlert then
		StaticPopupDialogs["NewAlert_Popup"] = {
			text = text,
			button1 = "Got it, thanks!",
			OnAccept = function() end,
			timeout = 0,
			whileDead = true,
			hideOnEscape = true,
			preferredIndex = 3,
		}
		StaticPopup_Show ("NewAlert_Popup")
		PlaySound(888)
		--message(text)
		end
        if ParagonSettings.ShowChat then
			print(text)
		end
    end
end)

-------------
-- Keyboard Shortcuts
-------------
SLASH_PARANNOUNCER1 = "/paragonannouncer"
SLASH_PARANNOUNCER2 = "/pa"
SlashCmdList["PARANNOUNCER"] = function()
    InterfaceOptionsFrame_OpenToCategory(AnnounceAdd.panel)
    InterfaceOptionsFrame_OpenToCategory(AnnounceAdd.panel)	-- Used twice as it only opens the window otherwise.
end 